@@ -1,4 +1,4 @@
-// Copyright (c) 2021, NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+// Copyright (c) 2021-2023, NVIDIA CORPORATION & AFFILIATES. All rights reserved.
 //
 // Licensed under the Apache License, Version 2.0 (the "License");
 // you may not use this file except in compliance with the License.
@@ -16,8 +16,9 @@
 #define DALI_CORE_MM_CUDA_VM_RESOURCE_H_
 
 #include <algorithm>
-#include <mutex>
+#include <cassert>
 #include <condition_variable>
+#include <mutex>
 #include <string>
 #include <utility>
 #include <vector>
@@ -30,6 +31,7 @@
 #include "dali/core/spinlock.h"
 #include "dali/core/mm/memory_resource.h"
 #include "dali/core/mm/detail/free_list.h"
+#include "dali/core/mm/pool_resource_base.h"
 #include "dali/core/small_vector.h"
 
 namespace dali {
@@ -39,7 +41,8 @@ namespace test {
 class VMResourceTest;
 }  // namespace test
 
-class cuda_vm_resource : public memory_resource<memory_kind::device> {
+class cuda_vm_resource : public memory_resource<memory_kind::device>,
+                         public pool_resource_base<memory_kind::device> {
  public:
   explicit cuda_vm_resource(int device_ordinal = -1,
                             size_t block_size = 0,
@@ -58,7 +61,7 @@ class cuda_vm_resource : public memory_resource<memory_kind::device> {
     purge();
   }
 
-  void *try_allocate_from_free(size_t size, size_t alignment) {
+  void *try_allocate_from_free(size_t size, size_t alignment) override {
     if (size == 0)
       return nullptr;
     adjust_params(size, alignment, true);
@@ -142,6 +145,54 @@ class cuda_vm_resource : public memory_resource<memory_kind::device> {
     }
   }
 
+  /**
+   * @brief Releases unused physical blocks
+   *
+   * Releases physical blocks that are currently allocated, but fully available.
+   */
+  void release_unused() override {
+    std::vector<cuvm::CUMem> blocks_to_free;
+    {
+      lock_guard pool_guard(pool_lock_);
+      mem_lock_guard mem_guard(mem_lock_);
+
+      ptrdiff_t num_blocks_to_free = 0;
+      for (auto &region : va_regions_) {
+        ptrdiff_t start = 0, end = 0;
+        for (;; start = end + 1) {
+          start = region.available.find(true, start);
+          end = region.available.find(false, start + 1);
+          if (end <= start)
+            break;
+          num_blocks_to_free += end - start;
+        }
+      }
+      blocks_to_free.reserve(num_blocks_to_free);
+
+      [&]() noexcept {  // this code mustn't throw!
+        for (auto &region : va_regions_) {
+          ptrdiff_t start = 0, end = 0;
+          for (;; start = end + 1) {
+            start = region.available.find(true, start);
+            end = region.available.find(false, start + 1);
+            if (end <= start)
+              break;
+            auto *start_ptr = region.block_ptr<char>(start);
+            auto *end_ptr = region.block_ptr<char>(end);
+            for (ptrdiff_t i = start; i < end; i++) {
+              blocks_to_free.push_back(region.unmap_block(i));
+              stat_.total_unmaps++;
+            }
+            free_mapped_.get_specific_block(start_ptr, end_ptr);
+          }
+        }
+      }();
+      assert(static_cast<ptrdiff_t>(blocks_to_free.size()) == num_blocks_to_free);
+      stat_.allocated_blocks -= blocks_to_free.size();
+    }
+    blocks_to_free.clear();  // free the physical memory
+  }
+
  protected:
   void do_deallocate(void *ptr, size_t size, size_t alignment) override {
     deallocate_impl(ptr, size, alignment, true);
