@@ -1,4 +1,4 @@
-// Copyright (c) 2017-2021, NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+// Copyright (c) 2017-2022, NVIDIA CORPORATION & AFFILIATES. All rights reserved.
 //
 // Licensed under the Apache License, Version 2.0 (the "License");
 // you may not use this file except in compliance with the License.
@@ -27,7 +27,6 @@
 #include "dali/core/permute.h"
 #include "dali/core/small_vector.h"
 #include "dali/core/dev_array.h"
-#include "dali/core/cuda_utils.h"
 #include "dali/core/format.h"
 
 namespace dali {
@@ -241,6 +240,19 @@ struct TensorShape<DynamicDimensions>
     return shape;
   }
 
+  /**
+   * @brief Return empty shape of specified dimensionality
+   */
+  static TensorShape<> empty_shape(int dim) {
+    assert(dim > 0);
+    TensorShape<> result;
+    result.resize(dim);
+    for (auto &elem : result) {
+      elem = 0;
+    }
+    return result;
+  }
+
   void resize(typename Base::size_type count) { shape.resize(count); }
 };
 
@@ -302,6 +314,19 @@ struct TensorShape : public TensorShapeBase<DeviceArray<int64_t, ndim>, ndim> {
     assert(count == ndim && "Not supported for count other than statically defined");
   }
 
+  /**
+   * @brief Return empty shape of specified dimensionality
+   */
+  static TensorShape<> empty_shape(int dim = ndim) {
+    assert(dim == ndim && "Not supported for count other than statically defined");
+    TensorShape<> result;
+    result.resize(dim);
+    for (auto &elem : result) {
+      elem = 0;
+    }
+    return result;
+  }
+
   static_assert(ndim >= 0, "TensorShape dimension should not be negative");
 };
 
@@ -663,7 +688,7 @@ struct TensorListShapeBase {
   }
 
 
-  ///@{
+  /** @{ */
   /**
    * Append this TensorListShape with shapes from other TensorListShapes.
    */
@@ -703,7 +728,7 @@ struct TensorListShapeBase {
   void append(const std::vector<TensorListShape<sample_ndim>> &tlss) {
     append(make_cspan(tlss.data(), tlss.size()));
   }
-  ///@}
+  /** @} */
 
  protected:
   int sample_dim() const { return static_cast<const Derived *>(this)->sample_dim(); }
@@ -769,9 +794,8 @@ struct TensorListShape<DynamicDimensions>
   TensorListShape &operator=(const TensorListShape &) = default;
   TensorListShape &operator=(TensorListShape &&other) {
     shapes = std::move(other.shapes);
-    nsamples = other.size();
-    ndim = other.sample_dim();
-    other.nsamples = 0;
+    nsamples = std::exchange(other.nsamples, 0);
+    ndim = other.sample_dim();  // Sample dim could be static, and it is valid to not zero it.
     return *this;
   }
 
@@ -786,9 +810,8 @@ struct TensorListShape<DynamicDimensions>
   template <int other_sample_ndim>
   TensorListShape &operator=(TensorListShape<other_sample_ndim> &&other) {
     shapes = std::move(other.shapes);
-    nsamples = other.size();
-    ndim = other.sample_dim();
-    other.nsamples = 0;
+    nsamples = std::exchange(other.nsamples, 0);
+    ndim = other.sample_dim();  // Sample dim could be static, and it is valid to not zero it.
     return *this;
   }
 
@@ -868,8 +891,7 @@ struct TensorListShape : TensorListShapeBase<TensorListShape<sample_ndim>, sampl
   TensorListShape &operator=(const TensorListShape &) = default;
   TensorListShape &operator=(TensorListShape &&other) {
     shapes = std::move(other.shapes);
-    nsamples = other.size();
-    other.nsamples = 0;
+    nsamples = std::exchange(other.nsamples, 0);
     return *this;
   }
 
